#include <system.h>

#define FLASH_CS	6

/* R}h̒` */
#define FLASH_WREN	0x06
#define FLASH_WRDI	0x04
#define FLASH_RDSR	0x05
#define FLASH_WRSR	0x01
#define FLASH_READ	0x03
#define FLASH_PROGRAM	0x02
#define FLASH_SECTOR_ERASE	0x52
#define FLASH_CHIP_ERASE	0x62
#define FLASH_RDID	0x15

void Flash_Init(void)
{
	sspstat = 0xC0;
	sspcon = 0x20;	//Enable serial port pins, clk idle state is low, SPI clk = fosc/4
	set_bit(portc, FLASH_CS);
}

/* SSPW[SPI[h0Ŏgp */
char Flash_Rw(char dat)
{
	sspbuf = dat;
	while(!(sspstat & (1<<BF)));
	return sspbuf;
}

/* ݋ */
void Flash_Wren(void)
{
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_WREN);//send cmd
	set_bit(portc, FLASH_CS);
}

/* ݋֎~ */
void Flash_Wrdi(void)
{
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_WRDI);//send cmd
	set_bit(portc, FLASH_CS);
}

/* Xe[^XWX^̓ǂݏo */
char Flash_Rdsr(void)
{
	char r_dat;
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_RDSR);//send cmd
	r_dat = Flash_Rw(0x00);
	set_bit(portc, FLASH_CS);
	return r_dat;
}

void Flash_Wrsr(char dat)
{
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_WRDI);//send cmd
	Flash_Rw(dat);
	set_bit(portc, FLASH_CS);
}

char Flash_Read_byte(int addr)
{
	char r_dat;
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_READ);//send cmd
	Flash_Rw(0x00);
	Flash_Rw((addr>>8)&0xFF);
	Flash_Rw(addr&0xFF);
	r_dat = Flash_Rw(0x00);
	set_bit(portc, FLASH_CS);
	return r_dat;
}

void Flash_Read_Str(int addr, char *r_dat, int times)
{
	int i;
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_READ);//send cmd
	Flash_Rw(0x00);
	Flash_Rw((addr>>8)&0xFF);
	Flash_Rw(addr&0xFF);
	for(i=0;i<times;i++)
	{
		*r_dat = Flash_Rw(0x00);
		r_dat++;
	}
	set_bit(portc, FLASH_CS);
}

/* `bv̑S̏ */
void Flash_ChipErase(void)
{
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_CHIP_ERASE);//send cmd
	set_bit(portc, FLASH_CS);
}

/* foCXID̓ǂݏo */
char Flash_Rdid(void)
{
	char r_dat;
	clear_bit(portc, FLASH_CS);
	Flash_Rw(FLASH_RDID);//send cmd
	r_dat=Flash_Rw(0x00);
	if(r_dat==0x1F)//ATMEL manufacturer code
	{
		r_dat=Flash_Rw(0x00);
	}
	else
	{
		r_dat=0xFF;
	}
	set_bit(portc, FLASH_CS);
	return r_dat;
}